require 'rake'
require 'rake/tasklib'
require 'tmpdir'

require 'razor'

module Razor
	
	module Rake
		
		module Git
			
			class BranchError < RuntimeError
				
				def initialize(branch)
					super("Must be on #{branch}")
				end
				
			end
			
			class CleanBranchError < RuntimeError
				
				def initialize
					super("Current branch must be clean.")
				end
				
			end
			
			class Generate < ::Rake::TaskLib
				
				attr_accessor :template_branch
				attr_accessor :site_branch
				attr_accessor :url_base
				
				def initialize(name='git:generate')
					@name = name
					@template_branch = 'master'
					@site_branch = 'site'
					@url_base = '/'
					yield self if block_given?
					define_task
				end
				
				def define_task
					task(@name, [:message]) { |t, args|
						ensure_on_branch template_branch
						ensure_branch_is_clean # needed ?
						# .gitinore workaround
						#TODO: Solve this problem
						gitignore = File.read '.gitignore' rescue ''
						Dir.mktmpdir { |tmp|
							Razor.generate(generate_options, '.', tmp)
							checkout site_branch do
								sh 'git rm -r .'
								sh "cp -r #{tmp}/* ."
								# .gitignore workaround
								File.open('.gitignore', 'w') {|f| f << gitignore }
								sh 'git add .'
								msg = args.message
								if msg.nil? or msg.empty?
									sh 'git commit'
								else
									sh "git commit -m #{msg}"
								end
							end
						}
					}
				end
				
				def current_branch
					`git status` =~ /^# On branch (.+)/
					return $1
				end
				
				def checkout(branch)
					if block_given?
						old = current_branch
						begin
							`git checkout #{branch}`
							yield
						ensure
							`git checkout #{old}`
						end
					else
						`git checkout #{branch}`
					end
				end
				
				def ensure_on_branch(branch)
					current_branch == branch and return
					ensure_branch_is_clean
					checkout branch
				rescue CleanBranchError
					raise BranchError.new(branch)
				end

				def ensure_branch_is_clean
					clean = (`git status` =~ /nothing to commit \(working directory clean\)\n$/ )
					clean and return
					raise CleanBranchError
				end
				
				def generate_options
					{
						:url_base => @url_base
					}
				end
				
			end
			
		end
		
	end
	
end
