module Razor
	
	class Renderer
		
		class Getter
			
			class << self
				
				attr_reader :renderers, :extensions
				
				def define_renderer(name, renderer)
					@renderers[name] = renderer
					class_eval(<<-END)
						def #{name}(string)
							self.class.renderers[#{name.inspect}].new(string)
						end
					END
				end
			
				def define_extension(name, renderer)
					@extensions[".#{name}"] = renderer
				end
				
			end
				
			@renderers = {}
			@extensions = {}
			
			attr :renderer
			
			def initialize(path, proc)
				@path = path
				if proc
					@renderer = instance_exec &proc
				else
					@renderer = instance_exec { nil }
				end
				if @renderer.is_a? String or @renderer.is_a? NilClass
					@renderer = Renderer.new(@renderer)
				end
				return @renderer
			end
			
			def from_file(filename)
				filepath = File.join(@path, filename)
				#str = (File.read(filepath, :mode=>'rb') rescue nil)
				begin
					str = File.open(filepath, 'rb') { |f| f.read }
				rescue
					str = nil
				end
				ext = File.extname filename
				renderer = self.class.extensions[ext]
				renderer ||= Renderer
				return renderer.new(str)
			end
			
		end
		
		class << self
		
			def get(path, proc)
				Getter.new(path, proc).renderer
			end
			
			def try_load
				constants.each { |constname|
					const = const_get constname
					const.is_a? Class or next
					const.ancestors.include? Renderer or next
					const.respond_to? :load or next
					begin
						const.load
					rescue LoadError
					end
				}
			end
			
			def name_ext(n)
				name(n)
				extension(n)
			end
			
			def name(n)
				Getter.define_renderer(n, self)
			end
			
			def extension(ext)
				Getter.define_extension(ext, self)
			end
		end
		
		attr :string
		
		def initialize(string)
			@string = string
		end
		
		def layout_render(context)
			valid? or return context['yield']
			render(context)
		end
		
		def valid?
			!!@string
		end
		
		def render(context)
			valid? or return ''
			Mustache.render(@string, context)
		end
		
		class BlueRenderer < Renderer
					
			def self.load
				require 'bluecloth'
				name_ext 'markdown'
				name_ext 'md'
			end
			
			def render(context)
				BlueCloth.new(super(context)).to_html
			end
			
		end
		
		class RedRenderer < Renderer
					
			def self.load
				require 'RedCloth'
				name_ext 'textile'
			end
			
			def render(context)
				RedCloth.new(super(context)).to_html
			end
			
		end
		
		class SassRenderer < Renderer
			
			def self.load
				require 'sass'
				name_ext 'sass'
			end
			
			def render(context)
				Sass::Engine.new(super(context), :syntax=>:sass ).render
			end
			
		end
		
		class ScssRenderer < Renderer
			
			def self.load
				require 'sass'
				name_ext 'scss'
			end
			
			def render(context)
				Sass::Engine.new(super(context), :syntax=>:scss ).render
			end
			
		end
		
	end
	
end